==> release.12.2.28.1.tgz: cgi-cpn/prontus_regenera_taxport.pl <==
#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------

# -------------------------------COMENTARIO GLOBAL---------------
# ---------------------------------------------------------------
# SCRIPT.
# -----------
# prontus_regenera_taxport.pl

# ---------------------------------------------------------------
# UBICACION.
# -----------
# /cgi-cpn/.

# ---------------------------------------------------------------
# PROPOSITO.
# -----------
# Script encargado de regenerar todas las portadas taxonómicas.
# Por un tema de performance, no debe usarse como cron. Debe usarse para eventuales
# migraciones u otros eventos por el estilo.
#
# Se crea este script para reemplazar el antiguo prontus_cron_taxport.pl sin parámetros,
# ya que por motivos de seguridad y performance, ya no cumple dicha funcion.
#
# $ARGV[0] : Nombre del Prontus (ej. prontus_noticias)
#
# ---------------------------------------------------------------

# ---------------------------------------------------------------
# DIRECTIVAS DE COMPILACION.
# ---------------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC, $pathLibsProntus);
}

##libs##
use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use DBI;
use File::Copy;
use POSIX;
use Time::HiRes;
use glib_dbi_03;
use glib_fildir_03;
use glib_hrfec_03;
use glib_str_03;
use lib_bd;
use lib_maxrunning;
use lib_tax;
use lib_utils;
##/libs##

my ($LOOP, %FORM, $NOM_PRONTUS, %TABLA_TEM, %TABLA_STEM, %TABLA_SECC);
my %NOMBASE_PLTS;
my ($FILASXPAG);
my %LEVELS2TRIGGER;
my %FIDS2PROCESS;

main: {
    lib_utils::setUtf8();
    # Soporta sólo 1 copia andando
    if (lib_maxrunning::maxExcedido(1)) {
        print "Error: Servidor ocupado. Intente otra vez mas tarde.\n";
        exit;
    }

    if ((!-d "$prontus_varglb::DIR_SERVER") || ($prontus_varglb::DIR_SERVER eq ''))  {
        print "Error: Document root no valido.\n";
        exit;
    }

    $FORM{'prontus'} = $ARGV[0];
    $FORM{'fid2process'} = $ARGV[1];

    if (scalar @ARGV < 1) {
        print "Parametros insuficientes Terminando\n";
        exit;
    }

    if ((!-d "$prontus_varglb::DIR_SERVER/$FORM{'prontus'}") || ($FORM{'prontus'} eq '')  || ($FORM{'prontus'} =~ /^\//))  {
        print "\nError: Directorio del publicador no es valido.";
        print "\nDebe indicar el nombre del Prontus a procesar (ej: prontus_noticias), como parametro de esta CGI\n";
        exit;
    }

    # valida el identificador de prontus
    if ($FORM{'prontus'} eq '' || !lib_utils::valida_prontus($FORM{'prontus'})) {
        print "El identificador de Prontus es inválido\n";
        exit;
    }

    # Carga variables de configuracion de prontus.
    my $msg_load = lib_utils::load_config_by_id($FORM{'prontus'});
    if ($msg_load ne '') {
        print STDERR "Error: $msg_load\n";
        exit;
    }

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'REGENERACION_TAX_TAG'} eq 'NO') {
        print "Error: La regeneración de portadas taxonómicas está deshabilitada en la configuración de Prontus.\n";
        exit;
    }

    # Se carga la configuracion local si existe
    lib_tax::carga_configuracion_local();

    # Se cargan todos los niveles, de manera inteligente
    &cargar_taxports();

    # Se ejecutan los procesos de regeneración.
    &lanzar_procesos();
}

# ---------------------------------------------------------------
sub conecta_db {
    my ($base, $msg_err_bd) = lib_bd::conectar_prontus_bd();

    if (!ref($base)) {
        print "ERROR: $msg_err_bd\n";

        exit;
    }

    $base->{mysql_auto_reconnect} = 1;

    return $base;
}

# ---------------------------------------------------------------
sub cargar_taxports {
    my $base = &conecta_db();

    # precarga tabla de secciones, temas, subtemas en hash global para uso posterior reiterado.
    %TABLA_SECC = lib_tax::carga_tabla_seccion($base);
    %TABLA_TEM = lib_tax::carga_tabla_temas($base);
    %TABLA_STEM = lib_tax::carga_tabla_subtemas($base);
    %FIDS2PROCESS = &get_fids2process($base);

    $base->disconnect;

    # secc
    foreach my $secc_id (keys %TABLA_SECC) {
        my ($secc_nom, $secc_port, $secc_nom4vistas) = split (/\t\t/, $TABLA_SECC{$secc_id});
        &agregar_taxports_thislevel($secc_id, '', '');

        # subtemas
        foreach my $temas_id (keys %TABLA_TEM) {
            my ($temas_nom, $temas_port, $temas_idparent, $temas_nom4vistas) = split (/\t\t/, $TABLA_TEM{$temas_id});
            next unless($secc_id eq $temas_idparent);
            &agregar_taxports_thislevel($secc_id, $temas_id, '');

            # subtemas
            foreach my $subtemas_id (keys %TABLA_STEM) {
                my ($subtemas_nom, $subtemas_port, $subtemas_idparent, $subtemas_nom4vistas) = split (/\t\t/, $TABLA_STEM{$subtemas_id});
                next unless($temas_id eq $subtemas_idparent);
                &agregar_taxports_thislevel($secc_id, $temas_id, $subtemas_id);
            }
        }
    }

    my @levels = keys %LEVELS2TRIGGER;
    my $totartics = (scalar @levels);

    if ($totartics == 0) {
        &agregar_taxports_thislevel('', '', ''); # todas
    }
}

# ---------------------------------------------------------------
# Genera todas las portadas tax (de la 1..n) correspondientes
# a este nivel taxonomico, para todas las vistas declaradas y fids.
sub agregar_taxports_thislevel {
    my ($secc_id, $temas_id, $subtemas_id) = @_;

    if ($subtemas_id) {
        $LEVELS2TRIGGER{"$secc_id/$temas_id/$subtemas_id"} = 1;
        $LEVELS2TRIGGER{"$secc_id/$temas_id/"} = 0;
        $LEVELS2TRIGGER{"$secc_id//"} = 0;

    } elsif($temas_id) {
        if (!exists $LEVELS2TRIGGER{"$secc_id/$temas_id/"}) {
            $LEVELS2TRIGGER{"$secc_id/$temas_id/"} = 1;
            $LEVELS2TRIGGER{"$secc_id//"} = 0;
        }
    } else {
        if (!exists $LEVELS2TRIGGER{"$secc_id//"}) {
            $LEVELS2TRIGGER{"$secc_id//"} = 1;
        }
    }
}

# ---------------------------------------------------------------
sub lanzar_procesos {
    my $rutaScript = $Bin;
    my $pathnice = lib_utils::get_path_nice();
    $pathnice = "$pathnice -n19 " if($pathnice);
    print STDERR "TAXPORT_MAX_MASTERS [$prontus_varglb::TAXPORT_MAX_MASTERS]\n";
    print STDERR "TAXPORT_MAX_WORKERS [$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_MAX_WORKERS'}]\n";

    foreach my $fid_name (keys %FIDS2PROCESS) { # key = 'fid_general:General(general.php)'
        foreach my $levels (sort keys %LEVELS2TRIGGER) {
            next unless($LEVELS2TRIGGER{$levels});

            if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_DINAMICAS'} eq 'NO') {
                while (lib_tax::check_taxport_running() >= $prontus_varglb::TAXPORT_MAX_MASTERS) {
                    sleep(5);
                }

                while (lib_tax::check_worker_running() >= ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_MAX_WORKERS'} + 1)) {
                    sleep(5);
                }

                my $param_especif_taxport = "$fid_name/$levels";
                my $cmd = "$pathnice $rutaScript/prontus_cron_taxport.pl $prontus_varglb::PRONTUS_ID $param_especif_taxport RG >/dev/null 2>&1 &";

                print "[" . glib_hrfec_03::get_dtime_pack4() . "] $cmd\n";

                system $cmd;

            } else {
                lib_tax::borrar_taxport("$fid_name/$levels");
            }

            Time::HiRes::usleep(100000);
        }
    }

    # procesamos las taxport de "all"
    foreach my $levels (sort keys %LEVELS2TRIGGER) {
        next unless($LEVELS2TRIGGER{$levels});

        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_DINAMICAS'} eq 'NO') {
            while (lib_tax::check_taxport_running() >= $prontus_varglb::TAXPORT_MAX_MASTERS) {
                sleep(5);
            }
            while (lib_tax::check_worker_running() >= ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_MAX_WORKERS'} + 1)) {
                sleep(5);
            }

            my $cmd = "$pathnice $rutaScript/prontus_cron_taxport.pl $prontus_varglb::PRONTUS_ID /$levels >/dev/null 2>&1 &";

            print "[" . glib_hrfec_03::get_dtime_pack4() . "] $cmd\n";

            system $cmd;
        } else {
            lib_tax::borrar_taxport("/$levels");
        }

        Time::HiRes::usleep(100000);
    }
}

# ---------------------------------------------------------------
# Obtiene fids para los cuales se generaran portadas taxonomicas.
# Estos son solo los que cuenten con plantilla taxonomica en el dir correspondiente:
# /<_prontus_id>/plantillas/tax/port/<_fid>[-<_mv>]/taxport[_<_seccion>][_<_tema>][_<_subtema>].<ext>
sub get_fids2process {
    my $base = $_[0];
    my %fids;
    my %fidswithtax;
    my $fid;
    lib_tax::load_taxport_fil();

    my $sql = "select ART_TIPOFICHA from ART where ART_IDSECC1<>0 or ART_IDSECC2<>0 or ART_IDSECC3<>0 group by ART_TIPOFICHA";
    my $salida = glib_dbi_03::ejecutar_sql_bind($base, $sql, \($fid));

    my $dir_plt = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_TEMPLATES'}$prontus_varglb::DIR_TAXPORT";

    while ($salida->fetch) {
        if (defined($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FORM_PLTS'}{$fid})) {
            if (&check_fid_fil($fid) || -d "$dir_plt/$fid") {
                $fidswithtax{$fid} = 1;
            }
        }
    }

    # Si viene por parametro el fid, se utiliza solo ese.
    if ($FORM{'fid2process'}) {
        if (!defined $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FORM_PLTS'}{$FORM{'fid2process'}}) {
            print "ERROR: El FID [$FORM{'fid2process'}] no existe en la configuracion de Prontus.\n";
            exit;
        }

        if (defined $fidswithtax{$FORM{'fid2process'}}) {
            return ($FORM{'fid2process'} => 1);
        } else {
            print "ERROR: No hay articulos con taxonomia con FID [$FORM{'fid2process'}].\n";
            exit;
        }
    }

    return %fidswithtax;
}

# ---------------------------------------------------------------
sub check_fid_fil {
    my $fid = $_[0];

    foreach my $filter (keys %{$lib_tax::CFG_FIL_TAXPORT{$prontus_varglb::PRONTUS_ID}}) {
        foreach my $fid_filtro (@{$lib_tax::CFG_FIL_TAXPORT{$prontus_varglb::PRONTUS_ID}{$filter}{'FIDS'}}) {
            if ($fid_filtro eq $fid) {
                return 1;
            }
        }
    }
    return 0;
}

==> release.12.3.1.2.tgz: cgi-cpn/prontus_regenera_taxport.pl <==
#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------

# -------------------------------COMENTARIO GLOBAL---------------
# ---------------------------------------------------------------
# SCRIPT.
# -----------
# prontus_regenera_taxport.pl

# ---------------------------------------------------------------
# UBICACION.
# -----------
# /cgi-cpn/.

# ---------------------------------------------------------------
# PROPOSITO.
# -----------
# Script encargado de regenerar todas las portadas taxonómicas.
# Por un tema de performance, no debe usarse como cron. Debe usarse para eventuales
# migraciones u otros eventos por el estilo.
#
# Se crea este script para reemplazar el antiguo prontus_cron_taxport.pl sin parámetros,
# ya que por motivos de seguridad y performance, ya no cumple dicha funcion.
#
# $ARGV[0] : Nombre del Prontus (ej. prontus_noticias)
#
# ---------------------------------------------------------------

# ---------------------------------------------------------------
# DIRECTIVAS DE COMPILACION.
# ---------------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC, $pathLibsProntus);
}

##libs##
use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use DBI;
use File::Copy;
use JSON;
use POSIX;
use Time::HiRes;
use glib_dbi_03;
use glib_fildir_03;
use glib_hrfec_03;
use glib_str_03;
use lib_bd;
use lib_lang;
use lib_maxrunning;
use lib_prontus;
use lib_tax;
use lib_utils;
##/libs##

my ($LOOP, %FORM, $NOM_PRONTUS, %TABLA_TEM, %TABLA_STEM, %TABLA_SECC);
my %NOMBASE_PLTS;
my ($FILASXPAG);
my %LEVELS2TRIGGER;
my %FIDS2PROCESS;

main: {
    lib_utils::setUp();

    # Soporta sólo 1 copia andando
    if (lib_maxrunning::maxExcedido(1)) {
        print lib_lang::get_message('global_error_server_busy_retry')."\n";
        exit;
    }

    if ((!-d "$prontus_varglb::DIR_SERVER") || ($prontus_varglb::DIR_SERVER eq ''))  {
        print lib_lang::get_message('global_docroot_invalid')."\n";
        exit;
    }

    $FORM{'prontus'} = $ARGV[0];
    $FORM{'fid2process'} = $ARGV[1];

    if (scalar @ARGV < 1) {
        print lib_lang::get_message('global_insufficient_arguments');
        exit;
    }

    if ((!-d "$prontus_varglb::DIR_SERVER/$FORM{'prontus'}") || ($FORM{'prontus'} eq '')  || ($FORM{'prontus'} =~ /^\//))  {
        print "\n".lib_lang::get_message('global_error_invalid_dir');
        print lib_lang::get_message('regenera_global_use');
        exit;
    }

    # valida el identificador de prontus
    if ($FORM{'prontus'} eq '' || !lib_utils::valida_prontus($FORM{'prontus'})) {
        print lib_lang::get_message('global_invalid_prontus_id') . "\n";
        exit;
    }

    # Carga variables de configuracion de prontus.
    my $msg_load = lib_utils::load_config_by_id($FORM{'prontus'});
    if ($msg_load ne '') {
        print STDERR "$msg_load\n";
        exit;
    }

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'REGENERACION_TAX_TAG'} eq 'NO') {
        print lib_lang::get_message('regenera_taxport_disabled')."\n";
        exit;
    }

    # Se carga la configuracion local si existe
    lib_tax::carga_configuracion_local();

    # Se cargan todos los niveles, de manera inteligente
    &cargar_taxports();

    # Se ejecutan los procesos de regeneración.
    &lanzar_procesos();
}

sub conecta_db {
    my ($base, $msg_err_bd) = lib_bd::conectar_prontus_bd();

    if (!ref($base)) {
        print $msg_err_bd, "\n";
        exit;
    }

    return $base;
}

sub cargar_taxports {
    my $base = &conecta_db();

    # precarga tabla de secciones, temas, subtemas en hash global para uso posterior reiterado.
    %TABLA_SECC = lib_tax::carga_hash_seccion($base);
    %TABLA_TEM = lib_tax::carga_hash_temas($base);
    %TABLA_STEM = lib_tax::carga_hash_subtemas($base);
    %FIDS2PROCESS = &get_fids2process($base);

    $base->disconnect;

    # secc
    foreach my $secc_id (keys %TABLA_SECC) {
        &agregar_taxports_thislevel($secc_id, '', '');

        # subtemas
        foreach my $temas_id (keys %TABLA_TEM) {
            next unless($secc_id eq $TABLA_TEM{$temas_id}{'id_padre'});
            &agregar_taxports_thislevel($secc_id, $temas_id, '');

            # subtemas
            foreach my $subtemas_id (keys %TABLA_STEM) {
                next unless($temas_id eq $TABLA_STEM{$subtemas_id}{'id_padre'});
                &agregar_taxports_thislevel($secc_id, $temas_id, $subtemas_id);
            }
        }
    }

    my @levels = keys %LEVELS2TRIGGER;
    my $totartics = (scalar @levels);

    if ($totartics == 0) {
        &agregar_taxports_thislevel('', '', ''); # todas
    }
}

# Genera todas las portadas tax (de la 1..n) correspondientes
# a este nivel taxonomico, para todas las vistas declaradas y fids.
sub agregar_taxports_thislevel {
    my ($secc_id, $temas_id, $subtemas_id) = @_;

    if ($subtemas_id) {
        $LEVELS2TRIGGER{"$secc_id/$temas_id/$subtemas_id"} = 1;
        $LEVELS2TRIGGER{"$secc_id/$temas_id/"} = 0;
        $LEVELS2TRIGGER{"$secc_id//"} = 0;

    } elsif($temas_id) {
        if (!exists $LEVELS2TRIGGER{"$secc_id/$temas_id/"}) {
            $LEVELS2TRIGGER{"$secc_id/$temas_id/"} = 1;
            $LEVELS2TRIGGER{"$secc_id//"} = 0;
        }
    } else {
        if (!exists $LEVELS2TRIGGER{"$secc_id//"}) {
            $LEVELS2TRIGGER{"$secc_id//"} = 1;
        }
    }
}

sub lanzar_procesos {
    my $rutaScript = $Bin;
    my $pathnice = lib_utils::get_path_nice();
    $pathnice = "$pathnice -n19 " if($pathnice);
    print STDERR "TAXPORT_MAX_MASTERS [$prontus_varglb::TAXPORT_MAX_MASTERS]\n";
    print STDERR "TAXPORT_MAX_WORKERS [$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_MAX_WORKERS'}]\n";

    foreach my $fid_name (keys %FIDS2PROCESS) { # key = 'fid_general:General(general.php)'
        foreach my $levels (sort keys %LEVELS2TRIGGER) {
            next unless($LEVELS2TRIGGER{$levels});

            if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_DINAMICAS'} eq 'NO') {
                while (lib_tax::check_taxport_running() >= $prontus_varglb::TAXPORT_MAX_MASTERS) {
                    sleep(5);
                }

                while (lib_tax::check_worker_running() >= ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_MAX_WORKERS'} + 1)) {
                    sleep(5);
                }

                my $param_especif_taxport = "$fid_name/$levels";
                my $cmd = "$pathnice $rutaScript/prontus_cron_taxport.pl $prontus_varglb::PRONTUS_ID $param_especif_taxport RG >/dev/null 2>&1 &";

                print "[" . glib_hrfec_03::get_dtime_pack4() . "] $cmd\n";

                system $cmd;

            } else {
                lib_tax::borrar_taxport("$fid_name/$levels");
            }

            Time::HiRes::usleep(100000);
        }
    }

    # procesamos las taxport de "all"
    foreach my $levels (sort keys %LEVELS2TRIGGER) {
        next unless($LEVELS2TRIGGER{$levels});

        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_DINAMICAS'} eq 'NO') {
            while (lib_tax::check_taxport_running() >= $prontus_varglb::TAXPORT_MAX_MASTERS) {
                sleep(5);
            }
            while (lib_tax::check_worker_running() >= ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_MAX_WORKERS'} + 1)) {
                sleep(5);
            }

            my $cmd = "$pathnice $rutaScript/prontus_cron_taxport.pl $prontus_varglb::PRONTUS_ID /$levels >/dev/null 2>&1 &";

            print "[" . glib_hrfec_03::get_dtime_pack4() . "] $cmd\n";

            system $cmd;
        } else {
            lib_tax::borrar_taxport("/$levels");
        }

        Time::HiRes::usleep(100000);
    }
}

# Obtiene fids para los cuales se generaran portadas taxonomicas.
# Estos son solo los que cuenten con plantilla taxonomica en el dir correspondiente:
# /<_prontus_id>/plantillas/tax/port/<_fid>[-<_mv>]/taxport[_<_seccion>][_<_tema>][_<_subtema>].<ext>
sub get_fids2process {
    my $base = $_[0];
    my %fids;
    my %fidswithtax;
    my $fid;
    lib_tax::load_taxport_fil();
    my $art_table = lib_utils::get_table('art');
    my $sql = "select art_tipoficha from $art_table where art_idsecc1<>0 or art_idsecc2<>0 or art_idsecc3<>0 group by art_tipoficha";
    my $salida = glib_dbi_03::ejecutar_sql_bind($base, $sql, \($fid));

    my $dir_plt = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_TEMPLATES'}$prontus_varglb::DIR_TAXPORT";

    while ($salida->fetch) {
        if (defined($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FORM_PLTS'}{$fid})) {
            if (&check_fid_fil($fid) || -d "$dir_plt/$fid") {
                $fidswithtax{$fid} = 1;
            }
        }
    }

    # Si viene por parametro el fid, se utiliza solo ese.
    if ($FORM{'fid2process'}) {
        if (!defined $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FORM_PLTS'}{$FORM{'fid2process'}}) {
            print lib_lang::get_interpolated_message('regenera_global_fid_missing', $FORM{'fid2process'});
            exit;
        }

        if (defined $fidswithtax{$FORM{'fid2process'}}) {
            return ($FORM{'fid2process'} => 1);
        } else {
            print lib_lang::get_interpolated_message('regenera_taxport_no_articles_for_fid', $FORM{'fid2process'});
            exit;
        }
    }

    return %fidswithtax;
}

sub check_fid_fil {
    my $fid = $_[0];

    foreach my $filter (keys %{$lib_tax::CFG_FIL_TAXPORT{$prontus_varglb::PRONTUS_ID}}) {
        foreach my $fid_filtro (@{$lib_tax::CFG_FIL_TAXPORT{$prontus_varglb::PRONTUS_ID}{$filter}{'FIDS'}}) {
            if ($fid_filtro eq $fid) {
                return 1;
            }
        }
    }
    return 0;
}
